/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core;

import java.security.GeneralSecurityException;
import javax.annotation.concurrent.ThreadSafe;
import net.java.truelicense.core.io.*;

/**
 * A caching license consumer manager which establishes a Chain Of
 * Responsibility with its parent license consumer manager.
 *
 * @author Christian Schlichtherle
 */
@ThreadSafe
abstract class ChainedLicenseConsumerManager
extends CachingLicenseConsumerManager implements LicenseProvider {

    /** The parent license consumer manager. */
    abstract LicenseConsumerManager parent();

    private boolean ftp() {
        return authentication().parameters().forSigning();
    }

    @Override
    public License install(Source source) throws GeneralSecurityException {
        try {
            return parent().install(source);
        } catch (final GeneralSecurityException primary) {
            if (ftp()) throw primary;
            return super.install(source);
        }
    }

    @Override public License view() throws GeneralSecurityException {
        try {
            return parent().view();
        } catch (final GeneralSecurityException primary) {
            try {
                return super.view(); // uses store()
            } catch (final GeneralSecurityException secondary) {
                synchronized (store()) {
                    try {
                        return super.view(); // repeat
                    } catch (final GeneralSecurityException ternary) {
                        return createIffNewFtp(ternary); // uses store(), too
                    }
                }
            }
        }
    }

    @Override public void verify() throws GeneralSecurityException {
        try {
            parent().verify();
        } catch (final GeneralSecurityException primary) {
            try {
                super.verify(); // uses store()
            } catch (final GeneralSecurityException secondary) {
                synchronized (store()) {
                    try {
                        super.verify(); // repeat
                    } catch (final GeneralSecurityException ternary) {
                        createIffNewFtp(ternary); // uses store(), too
                    }
                }
            }
        }
    }

    @Override public void uninstall() throws GeneralSecurityException {
        final LicenseConsumerManager parent = parent();
        try {
            parent.uninstall();
        } catch (final GeneralSecurityException primary) {
            if (ftp()) throw primary;
            super.uninstall();
        }
    }

    private License createIffNewFtp(final GeneralSecurityException ex)
    throws GeneralSecurityException {
        if (!ftp()) throw ex;
        final Store store = store();
        if (store.exists()) throw ex;
        return super.create(license(), store);
    }
}
