/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core;

import de.schlichtherle.license.LicenseContent;
import de.schlichtherle.xml.GenericCertificate;
import java.lang.reflect.Type;
import javax.annotation.concurrent.Immutable;
import net.java.truelicense.core.crypto.Encryption;
import net.java.truelicense.core.crypto.PbeParameters;
import net.java.truelicense.core.io.Transformation;
import net.java.truelicense.obfuscate.Obfuscate;

/**
 * The root context for the management of Version 1 (V1) format license keys.
 * This class is provided to enable applications to create, install, verify
 * and uninstall license keys in the format for TrueLicense 1.X applications.
 * Since TrueLicense 2.0, this format is obsolete and should not get used in
 * new applications.
 * Note that there is no interoperability between different format license keys.
 * <p>
 * Use this context to create a {@link LicenseVendorContext} or a
 * {@link LicenseConsumerContext}.
 * Here's an example for verifying the installed license key in a consumer
 * application:
 * <pre><code>
 * LicenseManagementContext mc = new V1LicenseManagementContext("MyApp 1");
 * LicenseConsumerContext cc = mc.consumer();
 * LicenseConsumerManager cm = cc.manager(...);
 * cm.verify();
 * </code></pre>
 * <p>
 * Or in a fluent API programming style:
 * <pre><code>
 * new V1LicenseManagementContext("MyApp 1").consumer().manager()...verify();
 * </code></pre>
 * <p>
 * Do not copy-paste this code!
 * Instead, use the TrueLicense Maven Archetype to generate a fully populated
 * template project for you.
 * <p>
 * Where required, this class should get subclassed to customize license
 * {@linkplain #license creation},
 * {@linkplain #initialization initialization}
 * or {@linkplain #validation validation},
 * {@linkplain #codec encoding} or to implement an authoritative
 * {@linkplain #clock clock} or to provide an alternative
 * {@linkplain #classLoader class loader} for loading key stores.
 * <p>
 * Note that this class is immutable.
 * Unless stated otherwise, all no-argument methods need to return consistent
 * objects so that caching them is not required.
 * A returned object is considered to be consistent if it compares
 * {@linkplain Object#equals(Object) equal} or at least behaves identical to
 * any previously returned object.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public class V1LicenseManagementContext
extends BasicLicenseManagementContext {

    private final LicenseXmlCodec codec = new LicenseXmlCodec();

    /**
     * Constructs a V1 license management context.
     * The provided string should get computed on demand from an obfuscated
     * form, e.g. by annotating a constant string value with the
     * {@literal @}{@link Obfuscate} annotation and processing it with the
     * TrueLicense Maven Plugin.
     *
     * @param subject the licensing subject, i.e. a product name with an
     *        optional version range, e.g. {@code MyApp 1}.
     */
    public V1LicenseManagementContext(String subject) { super(subject); }

    /** Returns a <em>new</em> license content. */
    // This introduces a cyclic dependency between the packages
    // de.schlichtherle.license and this package.
    // However, this is tolerable.
    @Override
    public LicenseContent license() { return new LicenseContent(); }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V1LicenseManagementContext}
     * returns a new {@link GenericCertificate}.
     */
    // This introduces a cyclic dependency between the packages
    // de.schlichtherle.license and this package.
    // However, this is tolerable.
    @Override public final GenericCertificate repository() {
        return new GenericCertificate();
    }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V1LicenseManagementContext}
     * returns a {@link LicenseXmlCodec}.
     */
    @Override public LicenseXmlCodec codec(Type expected) { return codec; }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V1LicenseManagementContext}
     * returns a compression for V1 format license keys.
     */
    @Override
    public final Transformation compression() {
        return new V1Compression();
    }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V1LicenseManagementContext}
     * returns an encryption for V1 format license keys with the given
     * parameters.
     */
    @Override protected final Encryption encryption(PbeParameters pbe) {
        return new V1Encryption(pbe);
    }
}
