/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core;

import java.lang.reflect.Type;
import javax.annotation.concurrent.Immutable;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import net.java.truelicense.core.codec.V2XmlCodec;
import net.java.truelicense.obfuscate.Obfuscate;

/**
 * The root context for the management of Version-2-with-XML (V2/XML) format
 * license keys.
 * Note that there is no interoperability between different format license keys.
 * <p>
 * Use this context to create a {@link LicenseVendorContext} or a
 * {@link LicenseConsumerContext}.
 * Here's an example for verifying the installed license key in a consumer
 * application:
 * <pre><code>
 * LicenseManagementContext mc = new V2XmlLicenseManagementContext("MyApp 1");
 * LicenseConsumerContext cc = mc.consumer();
 * LicenseConsumerManager cm = cc.manager(...);
 * cm.verify();
 * </code></pre>
 * <p>
 * Or in a fluent API programming style:
 * <pre><code>
 * new V2XmlLicenseManagementContext("MyApp 1").consumer().manager()...verify();
 * </code></pre>
 * <p>
 * Do not copy-paste this code!
 * Instead, use the TrueLicense Maven Archetype to generate a fully populated
 * template project for you.
 * <p>
 * Where required, this class should get subclassed to customize license
 * {@linkplain #license creation},
 * {@linkplain #initialization initialization}
 * or {@linkplain #validation validation},
 * {@linkplain #codec encoding} or to implement an authoritative
 * {@linkplain #clock clock} or to provide an alternative
 * {@linkplain #classLoader class loader} for loading key stores.
 * <p>
 * Note that this class is immutable.
 * Unless stated otherwise, all no-argument methods need to return consistent
 * objects so that caching them is not required.
 * A returned object is considered to be consistent if it compares
 * {@linkplain Object#equals(Object) equal} or at least behaves identical to
 * any previously returned object.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public class V2XmlLicenseManagementContext
extends BasicV2LicenseManagementContext {

    private volatile JAXBContext context;

    /**
     * Constructs a V2/XML license management context.
     * The provided string should get computed on demand from an obfuscated
     * form, e.g. by annotating a constant string value with the
     * {@literal @}{@link Obfuscate} annotation and processing it with the
     * TrueLicense Maven Plugin.
     *
     * @param subject the licensing subject, i.e. a product name with an
     *        optional version range, e.g. {@code MyApp 1}.
     */
    public V2XmlLicenseManagementContext(String subject) { super(subject); }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V2XmlLicenseManagementContext}
     * returns a new {@link V2XmlLicenseRepository}.
     */
    @Override public final V2XmlLicenseRepository repository() {
        return new V2XmlLicenseRepository();
    }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V2XmlLicenseManagementContext}
     * returns a {@link V2XmlCodec}.
     */
    @Override public V2XmlCodec codec(final Type expected) {
        try {
            return new V2XmlCodec(context());
        } catch (final JAXBException ex) {
            throw new IllegalStateException(ex);
        }
    }

    private JAXBContext context() {
        final JAXBContext c = context;
        return null != c ? c : (context = newContext());
    }

    /**
     * Returns a new JAXB context for use with {@linkplain #license licenses}
     * and {@linkplain #repository repositories}.
     * This method is normally only called once.
     * In a multi-threaded environment, it may get called more than once, but
     * then each invocation must return an object which behaves equivalent to
     * any previously returned object.
     */
    protected JAXBContext newContext() {
        try {
            return JAXBContext.newInstance(License.class, V2XmlLicenseRepository.class);
        } catch (final JAXBException ex) {
            throw new AssertionError(ex);
        }
    }
}
