/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.auth;

import java.nio.charset.Charset;
import java.security.*;
import java.util.Locale;
import javax.annotation.*;
import net.java.truelicense.core.codec.*;
import net.java.truelicense.core.io.*;
import net.java.truelicense.core.util.*;
import net.java.truelicense.obfuscate.Obfuscate;
import org.apache.commons.codec.binary.Base64;

/**
 * A generic repository.
 *
 * @author Christian Schlichtherle
 */
public abstract class BasicRepository implements Repository {

    @Obfuscate
    private static final String BASE64 = "base64";

    @Obfuscate
    private static final String SIGNATURE_TYPE = "application/octet-stream";

    @Obfuscate
    private static final String SIGNATURE_TRANSFER_ENCODING = BASE64;

    private static final Base64 base64 = new Base64();

    private @Nullable byte[] artifactData, signatureData;
    private @Nullable String signatureAlgorithm;

    // This is a volatile function of the codec.
    // So two threads may work with different objects, but because they are
    // function results, it doesn't matter as long as the codec compares equal.
    private volatile CachedCharset cc = new CachedCharset();

    /** Returns the codec for the setArtifact. */
    public abstract Codec codec();

    /**
     * Returns the character set as specified by - or implied from - the codec.
     * If this is {@code null}, then the codec uses an unknown character set or
     * does not produce text output and needs a Base64 transformation.
     */
    private Charset charset() {
        final Codec codec = codec();
        Charset c = cc.map(codec);
        if (null == c)
            this.cc = new CachedCharset(codec,
                    c = Codecs.charset(codec),
                    Long.MAX_VALUE);
        return c;
    }

    public String artifactType() { return codec().contentType(); }

    public String artifactTransferEncoding() {
        return null == charset() ? BASE64 : codec().contentTransferEncoding();
    }

    public @Nullable String getArtifact() {
        final byte[] artifactData = this.artifactData;
        return null == artifactData
                ? null
                : null == charset()
                    ? base64.encodeToString(artifactData)
                    : new String(artifactData, charset());
    }

    public void setArtifact(final @CheckForNull String body) {
        artifactData = null == body
                ? null
                : null == charset()
                    ? base64.decode(body)
                    : body.getBytes(charset());
    }

    public String signatureType() { return SIGNATURE_TYPE; }

    public String signatureTransferEncoding() {
        return SIGNATURE_TRANSFER_ENCODING;
    }

    public @Nullable String getSignature() {
        return base64.encodeToString(signatureData);
    }

    public void setSignature(final @CheckForNull String body) {
        signatureData = base64.decode(body);
    }

    public @Nullable String getSignatureAlgorithm()  {
        return signatureAlgorithm;
    }

    public void setSignatureAlgorithm(final @CheckForNull String algorithm) {
        this.signatureAlgorithm = algorithm;
    }

    @Override
    public <A> Artifactory<A> sign(
            final Signature engine,
            final PrivateKey key,
            final @CheckForNull A artifact)
    throws Exception {
        final MemoryStore store = new MemoryStore();
        final Codec codec = codec();
        codec.encode(store, artifact);
        final byte[] artifactData = store.data();
        engine.initSign(key);
        engine.update(artifactData);
        final byte[] signatureData = engine.sign();
        final String signatureAlgorithm = engine.getAlgorithm();
        this.artifactData = artifactData;
        this.signatureData = signatureData;
        this.signatureAlgorithm = signatureAlgorithm;
        return new EncodedArtifact<A>(codec, store);
    }

    /**
     * {@inheritDoc}
     *
     * @throws IllegalStateException if the properties do not match the
     *         properties used when signing.
     */
    @Override
    public Artifactory<?> verify(final Signature engine, final PublicKey key)
    throws Exception {
        if (!getSignatureAlgorithm().equalsIgnoreCase(engine.getAlgorithm()))
            throw new IllegalStateException();
        final Codec codec = codec();
        engine.initVerify(key);
        final byte[] artifactData = this.artifactData;
        engine.update(artifactData);
        if (!engine.verify(signatureData))
            throw new RepositoryIntegrityException();
        return new EncodedArtifact<Object>(codec, artifactData);
    }

    @Override
    public boolean equals(final Object obj) {
        if (this == obj) return true;
        if (!(obj instanceof BasicRepository)) return false;
        final BasicRepository that = (BasicRepository) obj;
        return  Objects.equals(this.getArtifact(), that.getArtifact()) &&
                Objects.equals(this.getSignature(), that.getSignature()) &&
                Strings.equalsIgnoreCase(this.getSignatureAlgorithm(), that.getSignatureAlgorithm());
    }

    @Override
    public int hashCode() {
        int c = 17;
        c = 31 * c + Objects.hashCode(getArtifact());
        c = 31 * c + Objects.hashCode(getSignature());
        c = 31 * c + Objects.hashCode(Strings.toLowerCase(getSignatureAlgorithm(), Locale.ROOT));
        return c;
    }

    private static class CachedCharset extends CacheEntry<Codec, Charset> {

        private static final long serialVersionUID = 1L;

        CachedCharset() { this(null, null, 0); } // => isObsolete() == true

        CachedCharset(
                @CheckForNull Codec codec,
                @CheckForNull Charset charset,
                long timeoutPeriodMillis) {
            super(codec, charset, timeoutPeriodMillis);
        }
    } // CharsetHolder
}
