/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.codec;

import java.beans.*;
import java.io.*;
import javax.annotation.Nullable;
import javax.annotation.concurrent.*;
import net.java.truelicense.core.io.*;
import net.java.truelicense.obfuscate.Obfuscate;

/**
 * A codec which encodes/decodes objects to/from XML with an
 * {@link XMLEncoder}/{@link XMLDecoder}.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public class V1XmlCodec implements Codec {

    @Obfuscate
    static final String CONTENT_TYPE = "application/xml; charset=utf-8";

    @Obfuscate
    static final String CONTENT_TRANSFER_ENCODING = "8bit";

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V1XmlCodec}
     * returns {@code "application/xml; charset=utf-8"}.
     *
     * @see <a href="http://tools.ietf.org/html/rfc3023">RFC 3023</a>
     */
    @Override public String contentType() { return CONTENT_TYPE; }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V1XmlCodec}
     * returns {@code "8bit"}.
     *
     * @see <a href="http://tools.ietf.org/html/rfc3023">RFC 3023</a>
     */
    @Override public String contentTransferEncoding() {
        return CONTENT_TRANSFER_ENCODING;
    }

    @Override public void encode(final Sink sink, final @Nullable Object obj)
    throws Exception {
        final ZeroToleranceListener ztl = new ZeroToleranceListener();
        final XMLEncoder enc = encoder(sink.output());
        try {
            enc.setExceptionListener(ztl);
            enc.writeObject(obj);
        } finally {
            enc.close(); // may pass exception to ztl!
        }
        ztl.check();
    }

    protected XMLEncoder encoder(OutputStream out) {
        return new XMLEncoder(out);
    }

    @Override public @Nullable Object decode(final Source source)
    throws Exception {
        final Object obj;
        final ZeroToleranceListener ztl = new ZeroToleranceListener();
        final XMLDecoder dec = decoder(source.input());
        try {
            dec.setExceptionListener(ztl);
            obj = dec.readObject();
        } finally {
            dec.close(); // may pass exception to ztl!
        }
        ztl.check();
        return obj;
    }

    protected XMLDecoder decoder(InputStream in) {
        return new XMLDecoder(in);
    }

    @NotThreadSafe
    private static class ZeroToleranceListener implements ExceptionListener {
        Exception ex;

        @Override
        public void exceptionThrown(final Exception ex) {
            assert null != ex;
            if (null == this.ex) this.ex = ex; // don't overwrite prior exception
        }

        void check() throws Exception { if (null != ex) throw ex; }
    } // ZeroToleranceListener
}
